@@ -50,9 +50,9 @@ options:
     version_added: "2.0"
   list:
     description:
-      - "Package name to run the equivalent of yum list C(--show-duplicates <package>) against. In addition to listing packages,
+      - "Package name to run the equivalent of yum list --show-duplicates <package> against. In addition to listing packages,
         use can also list the following: C(installed), C(updates), C(available) and C(repos)."
-      - This parameter is mutually exclusive with I(name).
+      - This parameter is mutually exclusive with C(name).
     type: str
   state:
     description:
@@ -98,7 +98,8 @@ options:
     version_added: "1.2"
   skip_broken:
     description:
-      - Skip packages with broken dependencies(devsolve) and are causing problems.
+      - Skip all unavailable packages or packages with broken dependencies
+        without raising an error. Equivalent to passing the --skip-broken option.
     type: bool
     default: "no"
     version_added: "2.3"
@@ -236,9 +237,32 @@ options:
     version_added: "1.5"
     default: "yes"
     type: bool
+  cacheonly:
+    description:
+      - Tells yum to run entirely from system cache; does not download or update metadata.
+    default: "no"
+    type: bool
+    version_added: "2.12"
+extends_documentation_fragment:
+- action_common_attributes
+- action_common_attributes.flow
+attributes:
+    action:
+        details: In the case of yum, it has 2 action plugins that use it under the hood, M(ansible.builtin.yum) and M(ansible.builtin.package).
+        support: partial
+    async:
+        support: none
+    bypass_host_loop:
+        support: none
+    check_mode:
+        support: full
+    diff_mode:
+        support: full
+    platform:
+        platforms: rhel
 notes:
-  - When used with a C(loop:) each package will be processed individually,
-    it is much more efficient to pass the list directly to the I(name) option.
+  - When used with a `loop:` each package will be processed individually,
+    it is much more efficient to pass the list directly to the `name` option.
   - In versions prior to 1.9.2 this module installed and removed each package
     given to the yum module separately. This caused problems when packages
     specified by filename or url had to be installed or removed together. In
@@ -370,6 +394,7 @@ EXAMPLES = '''
 '''
 
 from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils.common.locale import get_best_parsable_locale
 from ansible.module_utils.common.respawn import has_respawned, respawn_module
 from ansible.module_utils._text import to_native, to_text
 from ansible.module_utils.urls import fetch_url
@@ -592,11 +617,14 @@ class YumModule(YumDnf):
                 rpmbin = self.module.get_bin_path('rpm', required=True)
 
             cmd = [rpmbin, '-q', '--qf', qf, pkgspec]
+            if '*' in pkgspec:
+                cmd.append('-a')
             if self.installroot != '/':
                 cmd.extend(['--root', self.installroot])
             # rpm localizes messages and we're screen scraping so make sure we use
-            # the C locale
-            lang_env = dict(LANG='C', LC_ALL='C', LC_MESSAGES='C')
+            # an appropriate locale
+            locale = get_best_parsable_locale(self.module)
+            lang_env = dict(LANG=locale, LC_ALL=locale, LC_MESSAGES=locale)
             rc, out, err = self.module.run_command(cmd, environ_update=lang_env)
             if rc != 0 and 'is not installed' not in out:
                 self.module.fail_json(msg='Error from rpm: %s: %s' % (cmd, err))
@@ -933,7 +961,8 @@ class YumModule(YumDnf):
         else:
             res['changes'] = dict(installed=pkgs)
 
-        lang_env = dict(LANG='C', LC_ALL='C', LC_MESSAGES='C')
+        locale = get_best_parsable_locale(self.module)
+        lang_env = dict(LANG=locale, LC_ALL=locale, LC_MESSAGES=locale)
         rc, out, err = self.module.run_command(cmd, environ_update=lang_env)
 
         if rc == 1:
@@ -1008,7 +1037,7 @@ class YumModule(YumDnf):
                 # most common case is the pkg is already installed
                 envra = self.local_envra(package)
                 if envra is None:
-                    self.module.fail_json(msg="Failed to get nevra information from RPM package: %s" % spec)
+                    self.module.fail_json(msg="Failed to get envra information from RPM package: %s" % spec)
                 installed_pkgs = self.is_installed(repoq, envra)
                 if installed_pkgs:
                     res['results'].append('%s providing %s is already installed' % (installed_pkgs[0], package))
@@ -1221,48 +1250,47 @@ class YumModule(YumDnf):
 
     @staticmethod
     def parse_check_update(check_update_output):
-        updates = {}
-        obsoletes = {}
+        # preprocess string and filter out empty lines so the regex below works
+        out = '\n'.join((l for l in check_update_output.splitlines() if l))
 
-        # remove incorrect new lines in longer columns in output from yum check-update
-        # yum line wrapping can move the repo to the next line
-        #
-        # Meant to filter out sets of lines like:
+        # Remove incorrect new lines in longer columns in output from yum check-update
+        # yum line wrapping can move the repo to the next line:
         #  some_looooooooooooooooooooooooooooooooooooong_package_name   1:1.2.3-1.el7
         #                                                                    some-repo-label
-        #
-        # But it also needs to avoid catching lines like:
-        # Loading mirror speeds from cached hostfile
-        #
-        # ceph.x86_64                               1:11.2.0-0.el7                    ceph
+        out = re.sub(r'\n\W+(.*)', r' \1', out)
 
-        # preprocess string and filter out empty lines so the regex below works
-        out = re.sub(r'\n[^\w]\W+(.*)', r' \1', check_update_output)
-
-        available_updates = out.split('\n')
-
-        # build update dictionary
-        for line in available_updates:
+        updates = {}
+        obsoletes = {}
+        for line in out.split('\n'):
             line = line.split()
-            # ignore irrelevant lines
-            # '*' in line matches lines like mirror lists:
-            #      * base: mirror.corbina.net
-            # len(line) != 3 or 6 could be junk or a continuation
-            # len(line) = 6 is package obsoletes
-            #
-            # FIXME: what is  the '.' not in line  conditional for?
-
+            """
+            Ignore irrelevant lines:
+              - '*' in line matches lines like mirror lists: "* base: mirror.corbina.net"
+              - len(line) != 3 or 6 could be strings like:
+                  "This system is not registered with an entitlement server..."
+              - len(line) = 6 is package obsoletes
+              - checking for '.' in line[0] (package name) likely ensures that it is of format:
+                  "package_name.arch" (coreutils.x86_64)
+            """
             if '*' in line or len(line) not in [3, 6] or '.' not in line[0]:
                 continue
 
             pkg, version, repo = line[0], line[1], line[2]
             name, dist = pkg.rsplit('.', 1)
-            updates.update({name: {'version': version, 'dist': dist, 'repo': repo}})
+
+            if name not in updates:
+                updates[name] = []
+
+            updates[name].append({'version': version, 'dist': dist, 'repo': repo})
 
             if len(line) == 6:
                 obsolete_pkg, obsolete_version, obsolete_repo = line[3], line[4], line[5]
                 obsolete_name, obsolete_dist = obsolete_pkg.rsplit('.', 1)
-                obsoletes.update({obsolete_name: {'version': obsolete_version, 'dist': obsolete_dist, 'repo': obsolete_repo}})
+
+                if obsolete_name not in obsoletes:
+                    obsoletes[obsolete_name] = []
+
+                obsoletes[obsolete_name].append({'version': obsolete_version, 'dist': obsolete_dist, 'repo': obsolete_repo})
 
         return updates, obsoletes
 
@@ -1324,7 +1352,7 @@ class YumModule(YumDnf):
                     envra = self.local_envra(spec)
 
                     if envra is None:
-                        self.module.fail_json(msg="Failed to get nevra information from RPM package: %s" % spec)
+                        self.module.fail_json(msg="Failed to get envra information from RPM package: %s" % spec)
 
                     # local rpm files can't be updated
                     if self.is_installed(repoq, envra):
@@ -1341,7 +1369,7 @@ class YumModule(YumDnf):
                     envra = self.local_envra(package)
 
                     if envra is None:
-                        self.module.fail_json(msg="Failed to get nevra information from RPM package: %s" % spec)
+                        self.module.fail_json(msg="Failed to get envra information from RPM package: %s" % spec)
 
                     # local rpm files can't be updated
                     if self.is_installed(repoq, envra):
@@ -1403,22 +1431,64 @@ class YumModule(YumDnf):
         to_update = []
         for w in will_update:
             if w.startswith('@'):
+                # yum groups
                 to_update.append((w, None))
             elif w not in updates:
+                # There are (at least, probably more) 2 ways we can get here:
+                #
+                # * A virtual provides (our user specifies "webserver", but
+                #   "httpd" is the key in 'updates').
+                #
+                # * A wildcard. emac* will get us here if there's a package
+                #   called 'emacs' in the pending updates list. 'updates' will
+                #   of course key on 'emacs' in that case.
+
                 other_pkg = will_update_from_other_package[w]
+
+                # We are guaranteed that: other_pkg in updates
+                # ...based on the logic above. But we only want to show one
+                # update in this case (given the wording of "at least") below.
+                # As an example, consider a package installed twice:
+                # foobar.x86_64, foobar.i686
+                # We want to avoid having both:
+                #   ('foo*', 'because of (at least) foobar-1.x86_64 from repo')
+                #   ('foo*', 'because of (at least) foobar-1.i686 from repo')
+                # We just pick the first one.
+                #
+                # TODO: This is something that might be nice to change, but it
+                #       would be a module UI change. But without it, we're
+                #       dropping potentially important information about what
+                #       was updated. Instead of (given_spec, random_matching_package)
+                #       it'd be nice if we appended (given_spec, [all_matching_packages])
+                #
+                #       ... But then, we also drop information if multiple
+                #       different (distinct) packages match the given spec and
+                #       we should probably fix that too.
+                pkg = updates[other_pkg][0]
                 to_update.append(
                     (
                         w,
                         'because of (at least) %s-%s.%s from %s' % (
                             other_pkg,
-                            updates[other_pkg]['version'],
-                            updates[other_pkg]['dist'],
-                            updates[other_pkg]['repo']
+                            pkg['version'],
+                            pkg['dist'],
+                            pkg['repo']
                         )
                     )
                 )
             else:
-                to_update.append((w, '%s.%s from %s' % (updates[w]['version'], updates[w]['dist'], updates[w]['repo'])))
+                # Otherwise the spec is an exact match
+                for pkg in updates[w]:
+                    to_update.append(
+                        (
+                            w,
+                            '%s.%s from %s' % (
+                                pkg['version'],
+                                pkg['dist'],
+                                pkg['repo']
+                            )
+                        )
+                    )
 
         if self.update_only:
             res['changes'] = dict(installed=[], updated=to_update)
@@ -1444,7 +1514,8 @@ class YumModule(YumDnf):
         elif self.update_only:
             if pkgs['update']:
                 cmd = self.yum_basecmd + ['update'] + pkgs['update']
-                lang_env = dict(LANG='C', LC_ALL='C', LC_MESSAGES='C')
+                locale = get_best_parsable_locale(self.module)
+                lang_env = dict(LANG=locale, LC_ALL=locale, LC_MESSAGES=locale)
                 rc, out, err = self.module.run_command(cmd, environ_update=lang_env)
                 out_lower = out.strip().lower()
                 if not out_lower.endswith("no packages marked for update") and \
@@ -1454,7 +1525,8 @@ class YumModule(YumDnf):
                 rc, out, err = [0, '', '']
         elif pkgs['install'] or will_update and not self.update_only:
             cmd = self.yum_basecmd + ['install'] + pkgs['install'] + pkgs['update']
-            lang_env = dict(LANG='C', LC_ALL='C', LC_MESSAGES='C')
+            locale = get_best_parsable_locale(self.module)
+            lang_env = dict(LANG=locale, LC_ALL=locale, LC_MESSAGES=locale)
             rc, out, err = self.module.run_command(cmd, environ_update=lang_env)
             out_lower = out.strip().lower()
             if not out_lower.endswith("no packages marked for update") and \
@@ -1508,6 +1580,9 @@ class YumModule(YumDnf):
         if self.disable_excludes:
             self.yum_basecmd.extend(['--disableexcludes=%s' % self.disable_excludes])
 
+        if self.cacheonly:
+            self.yum_basecmd.extend(['--cacheonly'])
+
         if self.download_only:
             self.yum_basecmd.extend(['--downloadonly'])
 
