@@ -125,6 +125,13 @@ export ANSIBLE_CHECK_MODE_MARKERS=0
 
 run_test default
 
+# Check for async output
+# NOTE: regex to match 1 or more digits works for both BSD and GNU grep
+ansible-playbook -i inventory test_async.yml 2>&1 | tee async_test.out
+grep "ASYNC OK .* jid=[0-9]\{1,\}" async_test.out
+grep "ASYNC FAILED .* jid=[0-9]\{1,\}" async_test.out
+rm -f async_test.out
+
 # Hide skipped
 export ANSIBLE_DISPLAY_SKIPPED_HOSTS=0
 
@@ -204,5 +211,4 @@ rm -f meta_test.out
 
 # Ensure free/host_pinned non-lockstep strategies display correctly
 diff -u callback_default.out.free.stdout <(ANSIBLE_STRATEGY=free ansible-playbook -i inventory test_non_lockstep.yml 2>/dev/null)
-diff -u callback_default.out.fqcn_free.stdout <(ANSIBLE_STRATEGY=ansible.builtin.free ansible-playbook -i inventory test_non_lockstep.yml 2>/dev/null)
 diff -u callback_default.out.host_pinned.stdout <(ANSIBLE_STRATEGY=host_pinned ansible-playbook -i inventory test_non_lockstep.yml 2>/dev/null)
